library IEEE;

library modelsim_lib;
use modelsim_lib.util.ALL;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALl;
use std.textio.ALL;
use work.image1_pkg.ALL;
use work.image1_led_pkg.ALL;
use work.image1_wrappers_pkg.ALL;
use work.image1_top_tb_pkg.ALL;
use work.i2c_tb_pkg.ALL;
use work.i2c_slave_pkg.ALL;
use work.multiboot_pkg.ALL;

entity image1_top_tb is
end image1_top_tb;

architecture behavior of image1_top_tb is

  type t_tb_state is (ST_INIT, ST_READ_CTR0, ST_WRITE_LED, ST_READ_LED);


   constant c_CLK20M        : TIME := 50 ns;
   constant c_wb_clk_per    : time := 20 ns;
   --! ========================================================================
   --! Signals for the image1_top module
   --! ========================================================================
   signal s_RST_N           : STD_LOGIC := '1'; --! Never active
   signal s_CLK20_VCXO      : STD_LOGIC := '0';
   signal s_FPGA_CLK_P      : STD_LOGIC := '1';
   signal s_FPGA_CLK_N      : STD_LOGIC := '0';
   --! LEDs
   signal s_led_array_o     : t_led_array_o; -- STATUS LEDs
   signal s_pulse_led       : t_pulse_led_vector;
   signal s_pulse_i         : t_pulse_vector;
   signal s_pulse_o         : t_pulse_vector;
   signal s_inv_i           : STD_LOGIC_VECTOR(4 downto 1);
   signal s_inv_o           : STD_LOGIC_VECTOR(4 downto 1);
   --! Connections for i2c_slave_top
   signal s_sda_slave_oen   : STD_LOGIC;
   signal s_scl_slave_oen   : STD_LOGIC;
   signal s_I2C_master_i    : t_I2C_master_in;
   signal s_I2C_master_o    : t_I2C_master_out;
   signal s_I2C_slave_i     : t_I2C_slave_in;
   signal s_I2C_slave_o     : t_I2C_slave_out;
   signal s_FPGA_GA         : STD_LOGIC_VECTOR(4 downto 0);
   signal s_FPGA_GAP        : STD_LOGIC;
   --! Pins of the SPI interface to write into the Flash memory
   signal s_SPI_master_i    : t_SPI_master_in;
   signal s_SPI_master_o    : t_SPI_master_out;
   signal s_SPI_slave_i     : t_SPI_slave_in;
   signal s_SPI_slave_o     : t_SPI_slave_out;
   signal s_fpga_en         : t_fpga_en;
   signal level             : STD_LOGIC := '1';
   signal switch_i          : STD_LOGIC_VECTOR(1 downto 1);
   signal manual_rst_n_o    : STD_LOGIC;
   signal s_RTM_id_i        : t_RTM_id := c_RTM_id_default;
   signal rtmp, rtmm        : std_logic_vector(2 downto 0);
   
   --! ========================================================================
   --! Signals for the i2c_master_driver (Renesasa alike)
   --! ========================================================================
   signal s_i2c_addr_op     : STD_LOGIC_VECTOR(7 downto 0)
                                        := (others => '0');
   signal s_wishbone_addr   : STD_LOGIC_VECTOR(c_WB_ADDR_LENGTH - 1 downto 0)
                                        := (others => '0');
   signal s_wr_data         : STD_LOGIC_VECTOR(c_WR_DATA_LENGTH - 1 downto 0)
                                        := (others => '0');
   signal s_wr_data_tmp     : STD_LOGIC_VECTOR(c_WR_DATA_LENGTH - 1 downto 0)
                                        := (others => '0');
   signal s_rd_data         : STD_LOGIC_VECTOR(c_RD_DATA_LENGTH - 1 downto 0);

   signal s_I2C_driver_ctrl         : I2C_master_driver_ctrl
                                           := c_I2C_master_driver_ctrl_default;
   signal s_I2C_driver_ctrl_done    : I2C_master_driver_ctrl;

   signal wb_clk               : STD_LOGIC := '0';
   signal s_rst_SYS_A          : STD_LOGIC_VECTOR(c_RST_A_CLKS - 1 downto 0);
   signal s_rst                : STD_LOGIC;

   signal s_I2C_regs           : t_I2C_regs;
   signal s_MULTIBOOT_regs     : t_MULTIBOOT_regs;
   signal s_MULTIBOOT_regs_slv : t_MULTIBOOT_regs_slv;
   signal s_M25P32_regs        : t_M25P32_regs;
   signal s_M25P32_regs_slv    : t_M25P32_regs_slv;

   signal s_wr_fw_regs         : t_wr_fw_reg;
   signal s_wr_fw_regs_slv     : t_wr_fw_reg_slv;

   signal s_test    : STD_LOGIC;
   
   signal tb_state  : t_tb_state;
   
   signal gsr : std_logic;
   
begin

   s_MULTIBOOT_regs_slv <= f_STD_LOGIC_VECTOR(s_MULTIBOOT_regs);
   s_M25P32_regs_slv    <= f_STD_LOGIC_VECTOR(s_M25P32_regs);

   s_wr_fw_regs_slv     <= f_STD_LOGIC_VECTOR(s_wr_fw_regs);

   s_FPGA_GA            <= "11110";
   s_FPGA_GAP           <= '0';

   s_I2C_master_i       <= s_I2C_slave_o;
   s_I2C_slave_i        <= s_I2C_master_o;

   s_rst                <= s_rst_SYS_A(c_RST_A_CLKS - 1);


   --! Clock process definitions
   p_CLK20_VCXO :process
   begin
      s_CLK20_VCXO <= not(s_CLK20_VCXO);
        wait for c_CLK20M/2;
   end process;

   --! Clock process definitions
   p_FPGA_CLK :process
   begin
      s_FPGA_CLK_P <= not(s_FPGA_CLK_P);
      s_FPGA_CLK_N <= not(s_FPGA_CLK_N);
        wait for c_PLL_IN_PERIOD/2;
   end process;

   p_gsr: process
   begin
     gsr <= '1';
     wait for 200 ns;
     gsr <= '0';
     wait;
   end process p_gsr;

  p_rst: process
  begin
    s_rst_n <= '0';
    wait until gsr = '0';
    wait for 5 us;
    s_rst_n <= '1';
    wait;
  end process p_rst;

  p_wbclk: process
  begin
    wb_clk <= not wb_clk;
    wait for c_wb_clk_per;
  end process p_wbclk;


    --! @brief Process to bypass internal signals requiered for correct
    --!        use of i2c_master_driver
    p_sig_spy : process is
    begin
       init_signal_spy("/uut/s_clk_125MHz", "s_test", 1);
       init_signal_spy("/uut/inst_image1_core/s_clk.SYS_A", "wb_clk", 1);
       init_signal_spy("/uut/inst_image1_core/s_rst.SYS_A", "s_rst_SYS_A", 1);
       init_signal_spy("/uut/inst_image1_core/inst_i2c_slave/inst_i2c_regs/s_CTR0",
                                                "s_I2C_regs.CTR0", 1);
       init_signal_spy("/uut/inst_image1_core/inst_i2c_slave/inst_i2c_regs/s_LT",
                                                "s_I2C_regs.LT",   1);
       init_signal_spy("/uut/inst_image1_core/inst_i2c_slave/inst_i2c_slave_core/s_DRXA_slv",
                                                "s_I2C_regs.DRXA", 1);
       init_signal_spy("/uut/inst_image1_core/inst_i2c_slave/inst_i2c_slave_core/s_DRXB_slv",
                                                "s_I2C_regs.DRXB", 1);
       init_signal_spy("/uut/inst_image1_core/inst_i2c_slave/inst_i2c_slave_regs/s_DTX",
                                                "s_I2C_regs.DTX",  1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_CTR0",
                                                "s_MULTIBOOT_regs.CTR0", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_CTR1",
                                                "s_MULTIBOOT_regs.CTR1", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_STAT",
                                                "s_MULTIBOOT_regs.STAT", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_MBA",
                                                "s_MULTIBOOT_regs.MBA", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_GBA",
                                                "s_MULTIBOOT_regs.GBA", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_MBA",
                                                "s_MULTIBOOT_regs.MBA", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_GBA_ICAP",
                                                "s_MULTIBOOT_regs.GBA_ICAP", 1);
       init_signal_spy("/uut/inst_image1_core/inst_multiboot/multiboot_regs_inst/s_MBA_ICAP",
                                                "s_MULTIBOOT_regs.MBA_ICAP", 1);
       init_signal_spy("/uut/inst_image1_core/inst_m25p32/inst_m25p32_regs/s_FMI",
                                                "s_M25P32_regs.FMI", 1);
       init_signal_spy("/uut/inst_image1_core/inst_m25p32/inst_m25p32_regs/s_SR_m25p32",
                                                "s_M25P32_regs.SR_m25p32", 1);
       wait;
    end process p_sig_spy;


   -- Instantiate the Unit Under Test (UUT)

   i2c_driver: i2c_master_driver
   generic map(g_SCL_PERIOD    => c_SCL_I2C_DRIVER_PERIOD,
               g_LOG_PATH      => "../test/log/i2c_master_driver.txt")
   port map(tb_clk           => wb_clk,
            rst_n            => s_rst_n,

            sda_master_i     => s_I2C_master_i.SDA,
            sda_master_o     => s_I2C_master_o.SDA,
            scl_master_o     => s_I2C_master_o.SCL,

            i2c_addr_op_i    => s_i2c_addr_op,
            wishbone_addr_i  => s_wishbone_addr,
            wr_data_i        => s_wr_data,
            rd_data_o        => s_rd_data,

            start_i          => s_i2c_driver_ctrl.START,
            pause_i          => s_i2c_driver_ctrl.PAUSE,
            write_i          => s_i2c_driver_ctrl.WRITE,
            read_i           => s_i2c_driver_ctrl.READ,
            start_done_o     => s_i2c_driver_ctrl_done.START,
            pause_done_o     => s_i2c_driver_ctrl_done.PAUSE,
            write_done_o     => s_i2c_driver_ctrl_done.WRITE,
            read_done_o      => s_i2c_driver_ctrl_done.READ);

  rtmm <= "001";
  rtmp <= "000";
   uut: image1_top
   -- generic map(g_NUMBER_OF_CHANNELS => work.image1_top_tb_pkg.c_NUMBER_OF_CHANNELS)
   port map(RST_N                => s_RST_N,
            CLK20_VCXO           => s_CLK20_VCXO,
            FPGA_CLK_P           => s_FPGA_CLK_P,
            FPGA_CLK_N           => s_FPGA_CLK_N,
            LED_CTRL0            => s_led_array_o.CTRL0,
            LED_CTRL0_OEN        => s_led_array_o.CTRL0_OEN,
            LED_CTRL1            => s_led_array_o.CTRL1,
            LED_CTRL1_OEN        => s_led_array_o.CTRL1_OEN,
            LED_MULTICAST_2_0    => s_led_array_o.MULTICAST_2_0,
            LED_MULTICAST_3_1    => s_led_array_o.MULTICAST_3_1,
            LED_WR_GMT_TTL_TTLN  => s_led_array_o.WR_GMT_TTL_TTLN,
            LED_WR_LINK_SYSERROR => s_led_array_o.WR_LINK_SYSERROR,
            LED_WR_OK_SYSPW      => s_led_array_o.WR_OK_SYSPW,
            LED_WR_OWNADDR_I2C   => s_led_array_o.WR_OWNADDR_I2C,
            PULSE_FRONT_LED_N    => s_pulse_led.FRONT,
            PULSE_REAR_LED_N     => s_pulse_led.REAR,
            FPGA_INPUT_TTL_N     => s_pulse_i.FRONT,
            FPGA_OUT_TTL         => s_pulse_o.FRONT,
            FPGA_BLO_IN          => s_pulse_i.REAR,
            FPGA_TRIG_BLO        => s_pulse_o.REAR,
            INV_IN_N             => s_inv_i,
            INV_OUT              => s_inv_o,
            SCL_I                => s_I2C_master_o.SCL,
            SCL_O                => open,
            SCL_OE               => s_scl_slave_oen,
            SDA_I                => s_I2C_slave_i.SDA,
            SDA_O                => s_I2C_slave_o.SDA,
            SDA_OE               => s_sda_slave_oen,
            FPGA_GA              => s_FPGA_GA,
            FPGA_GAP             => s_FPGA_GAP,
            FPGA_PROM_CCLK       => s_SPI_master_o.CCLK,
            FPGA_PROM_CSO_B_N    => s_SPI_master_o.CSO_B_N,
            FPGA_PROM_DIN        => s_SPI_master_i.DIN,
            FPGA_PROM_MOSI       => s_SPI_master_o.MOSI,
            FPGA_OE              => s_fpga_en.GEN,
            FPGA_BLO_OE          => s_fpga_en.BLO,
            FPGA_TRIG_TTL_OE     => s_fpga_en.TTL,
            FPGA_INV_OE          => s_fpga_en.INV,
            LEVEL                => level,
            EXTRA_SWITCH         => switch_i,
            MR_N                 => manual_rst_n_o,
            fpga_rtmm_n_i        => rtmm,
            fpga_rtmp_n_i        => rtmp
            );


   --! Stimulus process
   p_i2c_tb: process

      procedure initial_inputs is
      begin
         s_pulse_i.REAR  <= (others => level);
         s_pulse_i.FRONT <= (others => level);
         s_inv_i         <= (others => level);
--         s_I2C_slave_i   <= c_I2C_master_out_default;
      end procedure;

      procedure start_I2C is
      begin
         s_I2C_driver_ctrl.START <= '1';
         assert false report "start condition to master" severity note;
           wait until rising_edge(s_I2C_driver_ctrl_done.START);
           wait until rising_edge(wb_clk);
         s_I2C_driver_ctrl.START <= '0';
      end procedure;


      --! This procedure is a wrapper in case we want to address with VME
      --! directions. Front padding is done with zeroes.
      procedure write_I2C(vme_slot   : std_logic_vector(4 downto 0);
            wb_addr : STD_LOGIC_VECTOR(s_wishbone_addr'length - 1 downto 0);
            wr_data : STD_LOGIC_VECTOR(s_wr_data'length       - 1 downto 0)) is
         variable v_i2c_addr  : STD_LOGIC_VECTOR(7 downto 0);
      begin
         v_i2c_addr(7 downto 1) := "10" & vme_slot; -- f_RENESAS_I2C_ADDRESSING(vme_slot);
         v_i2c_addr(0)          := '0';
           wait until rising_edge(wb_clk);
           -- i2c_master_driver
         s_i2c_addr_op     <= v_i2c_addr;
         s_wishbone_addr   <= wb_addr;
         s_wr_data         <= wr_data;
         s_I2C_driver_ctrl.WRITE <= '1';
           wait until rising_edge(wb_clk);
         s_I2C_driver_ctrl.WRITE <= '0';
           wait until rising_edge(s_I2C_driver_ctrl_done.WRITE);
           wait until rising_edge(wb_clk);
      end procedure;

      procedure read_I2C(vme_slot   : std_logic_vector(4 downto 0);
            wb_addr : STD_LOGIC_VECTOR(s_wishbone_addr'length - 1 downto 0)) is
         variable v_i2c_addr  : STD_LOGIC_VECTOR(7 downto 0);
      begin
         v_i2c_addr(7 downto 1) := "10" & vme_slot; -- f_RENESAS_I2C_ADDRESSING(vme_slot);
         v_i2c_addr(0)          := '0';
           wait until rising_edge(wb_clk);
           -- i2c_master_driver
         s_i2c_addr_op     <= v_i2c_addr;
         s_wishbone_addr   <= wb_addr;
         s_I2C_driver_ctrl.READ <= '1';
           wait until rising_edge(wb_clk);
         s_I2C_driver_ctrl.READ <= '0';
           wait until rising_edge(s_I2C_driver_ctrl_done.READ);
           wait until rising_edge(wb_clk);
      end procedure;

      variable v_iteration         : INTEGER := 0;
      variable v_m25p32_data_addr  : UNSIGNED(15 downto 0);
      variable v_m25p32_data       : UNSIGNED(31 downto 0);

   begin
     
     tb_state <= ST_INIT;
     
     wait until rising_edge(s_rst_n);
      initial_inputs;
--      wait until falling_edge(s_rst);
          wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
          
      --! First we place a write
      --! We read addr 1 (LT) via I2C
      --! Then, the core internally prefetchs via wishbone and then
      --! write the register into I2C to the master
      --! Please check in the wave.do file the internal register
      --! against the s_DTX register in the i2c_master_driver!
      

      --! [0] Reading I2C registers
      tb_state <= ST_READ_CTR0;
      
      start_I2C;
      assert false report "-------read ctr0" severity note;
      read_I2C(s_FPGA_GA, c_I2C_CTR0_addr);
      assert false report "-------done!" severity note;      
          assert false report "Read I2C CTR0"  severity note;
          assert    f_STD_LOGIC_VECTOR(s_I2C_regs.CTR0)
                  = s_rd_data report "Mismatch in I2C CTR0 readback"
                                               severity Error;
      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
      
      tb_state <= ST_WRITE_LED;
      
      start_i2c;
      assert false report "-------writing" severity note;
      write_i2c(s_fpga_ga, x"0080", x"00000055");
      assert false report "-------done!" severity note;
      
      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
      
      tb_state <= ST_READ_LED;
      
      start_i2c;
      assert false report "-------reading" severity note;
      read_i2c(s_fpga_ga, x"0080");
      assert false report "-------done!" severity note;
      
      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;

--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_I2C_DTX_addr);
--          assert False report "Reading I2C DTX" severity Warning;
--          assert    s_I2C_regs.DTX
--                  = s_rd_data report "Mismatch in I2C DTX readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_I2C_LT_addr);
--          assert False report "Read I2C LT"    severity Warning;
--      --! Commented because the status of LT should be read right
--      --! after the first received byte
--      --    assert    f_STD_LOGIC_VECTOR(s_I2C_regs.LT)
--      --            = s_rd_data report "Mismatch in I2C LT readback"
--      --                                         severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_I2C_DRXB_addr);
--          assert False report "Reading I2C DRXB" severity Warning;
--          assert    s_I2C_regs.DRXB
--                  = s_rd_data report "Mismatch in I2C DRXB readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_I2C_DRXA_addr);
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--          assert False report "Reading I2C DRXA" severity Warning;
--          assert    s_I2C_regs.DRXA
--                  = s_rd_data report "Mismatch in I2C DRXA readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--
--      --! [1] Writing-reading multiboot registers
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_CTR0_addr);
--          assert False report "Reading MULTIBOOT CTR0" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.CTR0
--                  = s_rd_data report "Mismatch in MULTIBOOT CTR0 readback"
--                                               severity Error;
--
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_CTR1_addr);
--          assert False report "Reading MULTIBOOT CTR1" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.CTR1
--                  = s_rd_data report "Mismatch in MULTIBOOT CTR1 readback"
--                                               severity Error;
--
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_STAT_addr);
--          assert False report "Reading MULTIBOOT STAT" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.STAT
--                  = s_rd_data report "Mismatch in MULTIBOOT STAT readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_MBA_addr);
--          assert False report "Reading MULTIBOOT MBA" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.MBA
--                  = s_rd_data report "Mismatch in MULTIBOOT MBA readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_GBA_addr);
--          assert False report "Reading MULTIBOOT GBA" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.GBA
--                  = s_rd_data report "Mismatch in MULTIBOOT GBA readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_MBA_ICAP_addr);
--          assert False report "Reading MULTIBOOT MBA_ICAP" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.MBA_ICAP
--                  = s_rd_data report "Mismatch in MULTIBOOT MBA_ICAP readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_MULTIBOOT_GBA_ICAP_addr);
--          assert False report "Reading MULTIBOOT GBA_ICAP" severity Warning;
--          assert    s_MULTIBOOT_regs_slv.GBA_ICAP
--                  = s_rd_data report "Mismatch in MULTIBOOT GBA_ICAP readback"
--                                               severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      s_wr_fw_regs.MULTIBOOT_CTR0         <= (OP       => OP_NONE,
--                                              PEN      => '0',
--                                              x        => (others => '0'));
--      start_I2C;
--      write_I2C(UNSIGNED(s_FPGA_GA),
--             c_MULTIBOOT_CTR0_addr,
--             s_wr_fw_regs_slv.MULTIBOOT_CTR0);
--          assert False report "Writing MULTIBOOT CTR0: OP_NONE" severity Warning;
--          assert    s_MULTIBOOT_regs.CTR0
--                  = s_wr_fw_regs.MULTIBOOT_CTR0
--                                   report "Mismatch in MULTIBOOT CTR0 write"
--                                   severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*50;
--
--      s_wr_fw_regs.MULTIBOOT_MBA      <= (ADDR     => to_unsigned(4096, 24),
--                                          SPI_OPC  => to_unsigned(0, 8));
--      s_wr_fw_regs.MULTIBOOT_GBA      <= (ADDR     => to_unsigned(32768, 24),
--                                          SPI_OPC  => to_unsigned(0, 8));
--
--      start_I2C;
--      write_I2C(UNSIGNED(s_FPGA_GA),
--                c_MULTIBOOT_MBA_addr,
--                s_wr_fw_regs_slv.MULTIBOOT_MBA);
--          assert False report "Writing MULTIBOOT MBA" severity Warning;
--          assert    s_MULTIBOOT_regs.MBA
--                  = s_wr_fw_regs.MULTIBOOT_MBA
--                                   report "Mismatch in MULTIBOOT MBA write"
--                                   severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      write_I2C(UNSIGNED(s_FPGA_GA),
--                c_MULTIBOOT_GBA_addr,
--                s_wr_fw_regs_slv.MULTIBOOT_GBA);
--          assert False report "Writing MULTIBOOT GBA" severity Warning;
--          assert    s_MULTIBOOT_regs.GBA
--                  = s_wr_fw_regs.MULTIBOOT_GBA
--                                   report "Mismatch in MULTIBOOT GBA write"
--                                   severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      s_wr_fw_regs.MULTIBOOT_CTR0         <= (OP       => OP_WR_MBA,
--                                              PEN      => '0',
--                                              x        => (others => '0'));
--      start_I2C;
--      write_I2C(UNSIGNED(s_FPGA_GA),
--             c_MULTIBOOT_CTR0_addr,
--             s_wr_fw_regs_slv.MULTIBOOT_CTR0);
--          assert False report "Writing MULTIBOOT CTR0: OP_WR_MBA" severity Warning;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*50;
--
--      s_wr_fw_regs.MULTIBOOT_CTR0         <= (OP       => OP_WR_GBA,
--                                              PEN      => '0',
--                                              x        => (others => '0'));
--      start_I2C;
--      write_I2C(UNSIGNED(s_FPGA_GA),
--             c_MULTIBOOT_CTR0_addr,
--             s_wr_fw_regs_slv.MULTIBOOT_CTR0);
--          assert False report "Writing MULTIBOOT CTR0: OP_WR_GBA" severity Warning;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*50;
--
--      s_wr_fw_regs.MULTIBOOT_CTR0         <= (OP       => OP_FULLMULTIBOOT,
--                                              PEN      => '0',
--                                              x        => (others => '0'));
--      start_I2C;
--      write_I2C(UNSIGNED(s_FPGA_GA),
--             c_MULTIBOOT_CTR0_addr,
--             s_wr_fw_regs_slv.MULTIBOOT_CTR0);
--          assert False report "Writing MULTIBOOT CTR0: OP_FULLMULTIBOOT"
--                                                                 severity Warning;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*50;











      --! [2] Writing-reading m25p32 registers
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_M25P32_FMI_addr);
--         assert False report "Reading M25P32 FMI" severity Warning;
--         assert    s_M25P32_regs_slv.FMI
--                 = s_rd_data report "Mismatch in M25P32 FMI readback"
--                                              severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      start_I2C;
--      read_I2C(UNSIGNED(s_FPGA_GA), c_M25P32_SR_m25p32_addr);
--         assert False report "Reading M25P32 SR_m25p32" severity Warning;
--         assert    s_M25P32_regs_slv.SR_m25p32
--                 = s_rd_data report "Mismatch in M25P32 SR_m25p32 readback"
--                                              severity Error;
--      wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      ----! TODO: these two checkings
--      --start_I2C;
--      --read_I2C(UNSIGNED(s_FPGA_GA), c_M25P32_DATA_READ_addr);
--      --wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--      --start_I2C;
--      --read_I2C(UNSIGNED(s_FPGA_GA), c_M25P32_DATA_WRITE_addr);
--      --wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--
--      --! c_M25P32_FMI_addr           [0200] W
--      --! c_M25P32_SR_m25p32_addr     [0204] W
--      --! c_M25P32_DATA_WRITE_addr    [0300 - 03FF] W
--      for v_iteration in 0 to 63 loop
--            v_m25p32_data      := to_unsigned(v_iteration*4, 32);
--            v_m25p32_data_addr := to_unsigned(v_iteration*4
--                         + to_integer(c_M25P32_DATA_WRITE_base_addr), 16);
--            s_wr_fw_regs.M25P32_DATA_WRITE <= STD_LOGIC_VECTOR(v_m25p32_data);
--         start_I2C;
--         write_I2C(UNSIGNED(s_FPGA_GA),
--                STD_LOGIC_VECTOR(v_m25p32_data_addr),
--                s_wr_fw_regs_slv.M25P32_DATA_WRITE);
--             assert False
--             report "Writing M25P32 data pool at position: "&INTEGER'image(
--                                                            4*v_iteration)
--                                                              severity Warning;
--         wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--         start_I2C;
--         read_I2C(UNSIGNED(s_FPGA_GA), STD_LOGIC_VECTOR(v_m25p32_data_addr));
--             assert False
--             report "Reading M25P32 data pool at position: "&INTEGER'image(
--                                                            4*v_iteration)
--                                                              severity Warning;
--             assert    s_wr_fw_regs.M25P32_DATA_WRITE
--                     = s_rd_data report "Mismatch in M25P32 data readback"
--                                                  severity Error;
--         wait for work.image1_pkg.c_WB_CLK_PERIOD*25;
--      end loop;
----
----      s_wr_fw_regs.M25P32_FMI             <= (OPR  => '0',
----                                              x    => (others => '0'),
----                                              OP   => x,
----                                              PG   => (others => '0'),
----                                              SCT  => (others => '0'),
----                                              y    => (others => '0'));
----
----      s_wr_fw_regs.M25P32_SR_m25p32       <= (WIP  => '0',
----                                              WEL  => '0',
----                                              BP0  => '0',
----                                              BP1  => '0',
----                                              BP2  => '0',
----                                              x    => (others => '0'),
----                                              SRWD => '0');
----
----      s_wr_fw_regs.M25P32_DATA_WRITE      <= (others => '0');
----
----
----
----   write_I2C(UNSIGNED(s_FPGA_GA),
----             c_M25P32_FMI_addr,
----             s_wr_fw_regs_slv.M25P32_FMI);
----   write_I2C(UNSIGNED(s_FPGA_GA),
----             c_M25P32_SR_m25p32_addr,
----             s_wr_fw_regs_slv.M25P32_SR_m25p32);
----   write_I2C(UNSIGNED(s_FPGA_GA),
----             c_M25P32_DATA_WRITE_addr,
----             s_wr_fw_regs_slv.M25P32_DATA_WRITE);

      assert False report "End of test" severity Failure;
   end process;

end;
