--==============================================================================
-- CERN (BE-CO-HT)
-- Testbench for glitch filter
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2013-03-12
--
-- version: 1.0
--
-- description: 
--
-- dependencies:
--
-- references:
-- 
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2013-03-12   Theodor Stana     t.stana@cern.ch     File created
--==============================================================================
-- TODO: - 
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity testbench is
end entity testbench;


architecture behav of testbench is

  --============================================================================
  -- Type declarations
  --============================================================================

  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_clk_per : time := 8 ns;
  constant c_reset_width : time := 31 ns;

  --============================================================================
  -- Component declarations
  --============================================================================
  component glitch_filt is
    generic
    (
      g_glitch_filt_len : natural := 4
    );
    port
    (
      clk_i : in  std_logic;
      rst_n_i : in  std_logic;

      dat_i : in  std_logic;
      dat_o : out std_logic
    );
  end component glitch_filt;

  component pulse_gen is
    generic
    (
      g_pwidth : natural := 200;
      g_freq   : natural := 400
    );
    port
    (
      clk_i   : in  std_logic;
      rst_n_i   : in  std_logic;
      pulse_o : out std_logic
    );
  end component pulse_gen;
  --============================================================================
  -- Signal declarations
  --============================================================================
  signal clk, rst_n, pulse, degl_out : std_logic := '0';

--==============================================================================
--  architecture begin
--==============================================================================
begin

  -- DUT INSTANTIATION
  cmp_dut: glitch_filt
    generic map
    (
      g_glitch_filt_len => 4
    )
    port map
    (
      clk_i   => clk,
      rst_n_i => rst_n,
      dat_i   => pulse,
      dat_o   => degl_out
    );

  -- PULSE GENERATOR FOR TRIGGER
  cmp_pulse_gen: pulse_gen
  generic map
  (
    g_pwidth => 6,
    g_freq   => 2000
  )
  port map
  (
    clk_i   => clk,
    rst_n_i => rst_n,
    pulse_o => pulse
  );

  -- CLOCK GENERATION
  p_clk: process
  begin
    clk <= not clk;
    wait for c_clk_per/2;
  end process p_clk;

  -- RESET GENERATION
  p_rst_n: process
  begin
    rst_n <= '0';
    wait for c_reset_width;
    rst_n <= '1';
    wait;
  end process p_rst_n;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
