--==============================================================================
-- CERN (BE-CO-HT)
-- Testbench
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2013-09-17
--
-- version: 1.0
--
-- description: 
--
-- dependencies:
--
-- references:
-- 
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2013-09-17   Theodor Stana     t.stana@cern.ch     File created
--==============================================================================
-- TODO: - 
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity testbench is
end entity testbench;


architecture arch of testbench is

  --============================================================================
  -- Type declarations
  --============================================================================

  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_clk_per : time := 8 ns;

  --============================================================================
  -- Component declarations
  --============================================================================
  component pulse_gen_gp is
    port
    (
      -- Input clock and active-low reset
      clk_i    : in  std_logic;
      rst_n_i  : in  std_logic;
  
      -- Active high enable signal
      en_i     : in  std_logic;
      
      -- Delay, pulse width and frequency inputs, in number of clk_i cycles
      delay_i  : in  std_logic_vector(31 downto 0);
      pwidth_i : in  std_logic_vector(31 downto 0);
      freq_i   : in  std_logic_vector(31 downto 0);
  
      -- Output pulse signal
      pulse_o  : out std_logic
    );
  end component pulse_gen_gp;

  --============================================================================
  -- Signal declarations
  --============================================================================
  signal clk, rst_n, en : std_logic := '0';
  signal del, del1, pw, f : std_logic_vector(31 downto 0);
  signal pulse, pulse1 : std_logic;


--==============================================================================
--  architecture begin
--==============================================================================
begin

  UUT: pulse_gen_gp
    port map
    (
      -- Input clock and active-low reset
      clk_i    => clk,
      rst_n_i  => rst_n,
  
      -- Active high enable signal
      en_i     => en,
      
      -- Delay, pulse width and frequency inputs, in number of clk_i cycles
      delay_i  => del,
      pwidth_i => pw,
      freq_i   => f,
  
      -- Output pulse signal
      pulse_o  => pulse
    );

  UUT1: pulse_gen_gp
    port map
    (
      -- Input clock and active-low reset
      clk_i    => clk,
      rst_n_i  => rst_n,
  
      -- Active high enable signal
      en_i     => en,
      
      -- Delay, pulse width and frequency inputs, in number of clk_i cycles
      delay_i  => del1,
      pwidth_i => pw,
      freq_i   => f,
  
      -- Output pulse signal
      pulse_o  => pulse1
    );

  p_clk : process is
  begin
    clk <= not clk;
    wait for c_clk_per/2;
  end process p_clk;

  p_rst : process is
  begin
    rst_n <= '0';
    wait for 120 ns;
    rst_n <= '1';
    wait;
  end process;

  p_stim : process is
  begin
    en <= '0';
    pw <= (others => '0');
    del <= (others => '0');
    f <= (others => '0');
    wait until rst_n = '1';

    pw <= x"00000096";
    del <= (others => '0');
    del1 <= x"0000007d";
    f <= x"000004e2";
    wait for 31ns;
    en <= '1';
    wait for 200 us;
    f <= x"000009c4";
    --en <= '0';
    wait for 100 us;
    f <= x"00001388";
    --en <= '1';
    wait for 200 us;
    en <= '0';
    wait for 10 us;
    f <= x"000009c4";
    en <= '1';
    wait for 100 us;
    f <= x"000004e2";
    wait for 1 us;
    en <= '0';
    wait for 100 us;
    en <= '1';
    wait for 100 us;
    en <= '0';
    wait;
  end process p_stim;

end architecture arch;
--==============================================================================
--  architecture end
--==============================================================================
